from __future__ import annotations

import os
from pathlib import Path
from typing import Tuple

from buildguard.process import run_command


def create_virtual_environment(venv_path: Path, python_executable: str) -> None:
    command_result = run_command([python_executable, '-m', 'venv', str(venv_path)])
    if command_result.return_code != 0:
        message = command_result.stderr.strip() or command_result.stdout.strip() or 'unknown error'
        raise RuntimeError(f'failed to create virtual environment: {message}')


def get_venv_executables(venv_path: Path) -> Tuple[Path, Path]:
    if os.name == 'nt':
        scripts_directory = venv_path / 'Scripts'
        python_path = scripts_directory / 'python.exe'
        pip_path = scripts_directory / 'pip.exe'
    else:
        bin_directory = venv_path / 'bin'
        python_path = bin_directory / 'python'
        pip_path = bin_directory / 'pip'
    return python_path, pip_path



from __future__ import annotations

import re
from typing import Dict, Iterable, List, Optional, Tuple

_MEANINGFUL_LINE_LIMIT = 10
_NOISE_PREFIXES = (
    'WARNING: Retrying',
    'Looking in indexes:',
    '[notice] ',
)
_SIGNAL_PATTERNS = (
    'Failed building wheel for',
    'error: subprocess-exited-with-error',
    'Could not find a version that satisfies the requirement',
    'No matching distribution found for',
    'Preparing metadata (pyproject.toml) ... error',
    'Getting requirements to build wheel ... error',
    'ResolutionImpossible',
    'metadata-generation-failed',
    'Failed to build',
    'ERROR:',
)
_MAX_SUMMARY_LINE_LENGTH = 220


def _meaningful_lines(lines: Iterable[str]) -> List[str]:
    return [line.strip() for line in lines if line and line.strip()]


def _normalize_name(package_name: str) -> str:
    return re.sub(r'[-_.]+', '-', package_name.strip().lower())


def _extract_collecting_package(line: str) -> Optional[str]:
    collecting_match = re.search(r'^Collecting\s+(.+)$', line)
    if collecting_match is None:
        return None
    return collecting_match.group(1).strip()


def _extract_package_name_from_spec(package_spec: str) -> str:
    stripped_spec = package_spec.strip()
    for separator in ['==', '>=', '<=', '!=', '~=', '>', '<']:
        if separator in stripped_spec:
            return stripped_spec.split(separator, 1)[0].strip()
    return stripped_spec.split('[', 1)[0].strip()


def extract_package_name_from_requirement_spec(package_spec: str) -> str:
    return _extract_package_name_from_spec(package_spec)


def extract_failing_package_hint(output_text: str) -> Tuple[Optional[str], bool]:
    lines = _meaningful_lines(output_text.splitlines())
    latest_collecting_by_name: Dict[str, str] = {}
    most_recent_collecting_spec: Optional[str] = None

    for line in lines:
        collecting_spec = _extract_collecting_package(line)
        if collecting_spec is not None:
            collecting_name = _extract_package_name_from_spec(collecting_spec)
            latest_collecting_by_name[_normalize_name(collecting_name)] = collecting_spec
            most_recent_collecting_spec = collecting_spec
            continue

        no_version_match = re.search(
            r'Could not find a version that satisfies the requirement\s+([^\s]+)',
            line,
        )
        if no_version_match is not None:
            return no_version_match.group(1).strip(), False

        no_distribution_match = re.search(
            r'No matching distribution found for\s+([^\s]+)',
            line,
        )
        if no_distribution_match is not None:
            return no_distribution_match.group(1).strip(), False

        failed_wheel_match = re.search(r'Failed building wheel for\s+([^\s]+)$', line)
        if failed_wheel_match is not None:
            package_name = failed_wheel_match.group(1).strip()
            normalized_name = _normalize_name(package_name)
            if normalized_name in latest_collecting_by_name:
                return latest_collecting_by_name[normalized_name], False
            return package_name, False

        building_wheel_match = re.search(r'Building wheel for\s+([^\s]+)', line)
        if building_wheel_match is not None:
            package_name = building_wheel_match.group(1).strip()
            normalized_name = _normalize_name(package_name)
            if normalized_name in latest_collecting_by_name:
                latest_collecting_by_name[normalized_name] = latest_collecting_by_name[normalized_name]
            else:
                latest_collecting_by_name[normalized_name] = package_name
            most_recent_collecting_spec = latest_collecting_by_name[normalized_name]
            continue

        # Fallback: pip sometimes reports only generic errors. Use the most recently
        # collected requirement as a best-effort hint.
        if (
            'subprocess-exited-with-error' in line
            or 'metadata-generation-failed' in line
            or 'Preparing metadata (pyproject.toml) ... error' in line
            or 'Getting requirements to build wheel ... error' in line
        ):
            if most_recent_collecting_spec is not None:
                return most_recent_collecting_spec, True

    return None, False


def extract_error_tail(stdout_text: str, stderr_text: str) -> Tuple[str, ...]:
    combined_lines = _meaningful_lines((stdout_text + '\n' + stderr_text).splitlines())
    filtered_lines = [line for line in combined_lines if not line.startswith(_NOISE_PREFIXES)]
    source_lines = filtered_lines if filtered_lines else combined_lines

    summarized_lines = [_summarize_error_line(line) for line in source_lines]
    signal_lines = [line for line in summarized_lines if _is_signal_line(line)]
    selected_lines = signal_lines if signal_lines else summarized_lines
    return tuple(selected_lines[-_MEANINGFUL_LINE_LIMIT:])


def extract_stream_tail(stream_text: str, limit: int = _MEANINGFUL_LINE_LIMIT) -> Tuple[str, ...]:
    meaningful = _meaningful_lines(stream_text.splitlines())
    return tuple(meaningful[-limit:])


def infer_failure_diagnostics(
    output_text: str,
    failing_package_hint: Optional[str],
) -> Tuple[Optional[str], Optional[str], Tuple[str, ...]]:
    normalized_output = output_text.lower()

    if (
        'resolutionimpossible' in normalized_output
        or 'cannot install' in normalized_output and 'because these package versions have conflicting dependencies' in normalized_output
    ):
        return (
            'resolver_conflict',
            'Dependency resolution failed because pinned packages have incompatible constraints.',
            (
                'Review transitive constraints for conflicting pins and align incompatible versions.',
                'Temporarily loosen one or more strict pins to find a compatible set.',
            ),
        )

    if 'failed building wheel for' in normalized_output:
        package_text = f' for {failing_package_hint}' if failing_package_hint else ''
        return (
            'wheel_build_failure',
            f'pip could not build a wheel{package_text}.',
            (
                'Prefer a package version that ships prebuilt wheels for this Python version/platform.',
                'Install required native build tooling and headers, then retry.',
            ),
        )

    if (
        'metadata-generation-failed' in normalized_output
        or 'preparing metadata (pyproject.toml) ... error' in normalized_output
        or 'getting requirements to build wheel ... error' in normalized_output
    ):
        package_text = f' for {failing_package_hint}' if failing_package_hint else ''
        return (
            'metadata_generation_failure',
            f'Build metadata generation failed{package_text}.',
            (
                'This often indicates old package pins that are incompatible with modern build backends.',
                'Try a newer compatible version of the failing package or pin older build tooling in that package path.',
            ),
        )

    if (
        'no matching distribution found for' in normalized_output
        or 'could not find a version that satisfies the requirement' in normalized_output
    ):
        package_text = f' for {failing_package_hint}' if failing_package_hint else ''
        return (
            'missing_distribution',
            f'No installable distribution was found{package_text} on the current index/Python/platform.',
            (
                'Verify that the pinned version exists on your package index.',
                'Check Python-version/platform compatibility for that package version.',
            ),
        )

    if 'requires-python' in normalized_output or 'is not a supported wheel on this platform' in normalized_output:
        return (
            'python_platform_incompatibility',
            'At least one dependency is incompatible with the current Python version or platform.',
            (
                'Use a compatible Python interpreter version for this dependency set.',
                'Update pins to versions that support your runtime platform.',
            ),
        )

    return (
        None,
        None,
        tuple(),
    )


def parse_available_versions_from_pip_index_output(output_text: str) -> Tuple[str, ...]:
    for line in output_text.splitlines():
        stripped_line = line.strip()
        if stripped_line.startswith('Available versions:'):
            versions_text = stripped_line.split(':', 1)[1].strip()
            if not versions_text:
                return tuple()
            return tuple(version.strip() for version in versions_text.split(',') if version.strip())
    return tuple()


def _summarize_error_line(line: str) -> str:
    summarized_line = line
    if ' (from versions:' in summarized_line:
        summarized_line = summarized_line.split(' (from versions:', 1)[0]
    if len(summarized_line) > _MAX_SUMMARY_LINE_LENGTH:
        summarized_line = summarized_line[: _MAX_SUMMARY_LINE_LENGTH - 3] + '...'
    return summarized_line


def _is_signal_line(line: str) -> bool:
    for pattern in _SIGNAL_PATTERNS:
        if pattern in line:
            return True
    return False



from __future__ import annotations

import argparse
import json
import logging
import os
import sys
import threading
import time
from itertools import cycle
from typing import Optional, Sequence

from buildguard import __version__
from buildguard.checker import run_check
from buildguard.licensing import LICENSE_NOTICE, should_show_license_notice
from buildguard.report import format_json_report, format_text_report

logger = logging.getLogger(__name__)


class _ProgressSpinner:
    def __init__(self, message: str, enabled: bool) -> None:
        self._message = message
        self._enabled = enabled
        self._stop_event = threading.Event()
        self._thread: Optional[threading.Thread] = None

    def start(self) -> None:
        if not self._enabled:
            return
        self._thread = threading.Thread(target=self._spin, daemon=True)
        self._thread.start()

    def stop(self) -> None:
        if not self._enabled:
            return
        self._stop_event.set()
        if self._thread is not None:
            self._thread.join()
        sys.stderr.write('\r' + (' ' * 100) + '\r')
        sys.stderr.flush()

    def _spin(self) -> None:
        for spinner_char in cycle('|/-\\'):
            if self._stop_event.is_set():
                break
            sys.stderr.write(f'\r{self._message} {spinner_char}')
            sys.stderr.flush()
            time.sleep(0.1)


def _build_epilog() -> str:
    epilog = (
            'check command options:\n'
            '  --json\n'
            '  --timeout <seconds>\n'
            '  --python <executable>\n'
            '  --python-m-pip\n'
            '  --keep-venv\n'
            '  --no-upgrade-tools\n'
            '  --pip-version <version>\n'
            '  --setuptools-version <version>\n'
            '  --wheel-version <version>\n'
            '  --verbose-errors\n'
            '  --show-available-versions'
    )
    if should_show_license_notice():
        epilog += f'\n\n{LICENSE_NOTICE}'
    return epilog


def _maybe_write_license_notice() -> None:
    if should_show_license_notice():
        sys.stdout.write(f'{LICENSE_NOTICE}\n')


def _build_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog='buildguard',
        description='Catch upstream Python dependency breakage before it surprises your CI build.',
        epilog=_build_epilog(),
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('--version', action='version', version=f'%(prog)s {__version__}')

    subparsers = parser.add_subparsers(dest='command', required=True)
    check_parser = subparsers.add_parser(
        'check',
        help='check for upstream dependency drift by installing requirements in a clean virtual environment',
    )
    check_parser.add_argument('requirements_path', help='path to requirements.txt file')
    check_parser.add_argument('--json', action='store_true', dest='emit_json', help='emit JSON output')
    check_parser.add_argument('--timeout', type=int, default=300, help='timeout in seconds for pip install')
    check_parser.add_argument('--python', default=sys.executable, dest='python_executable', help='python executable used to create virtual environment')
    check_parser.add_argument(
        '--python-m-pip',
        action='store_true',
        help='invoke pip as "python -m pip" inside the temporary virtual environment',
    )
    check_parser.add_argument('--keep-venv', action='store_true', help='do not delete temporary virtual environment')
    check_parser.add_argument('--no-upgrade-tools', action='store_true', help='skip upgrading pip, setuptools, and wheel')
    check_parser.add_argument('--pip-version', default=None, help='pin pip version for tool upgrade step')
    check_parser.add_argument('--setuptools-version', default=None, help='pin setuptools version for tool upgrade step')
    check_parser.add_argument('--wheel-version', default=None, help='pin wheel version for tool upgrade step')
    check_parser.add_argument(
        '--verbose-errors',
        action='store_true',
        help='include additional pip stdout/stderr tail lines in text output',
    )
    check_parser.add_argument(
        '--show-available-versions',
        action='store_true',
        help='on missing-distribution failures, show a short list of available versions',
    )
    check_parser.description = (
        'Create a clean virtual environment and install a pinned dependency set to detect upstream '
        'ecosystem drift before your main CI build runs.'
    )

    return parser


def main(argv: Optional[Sequence[str]] = None) -> int:
    parser = _build_parser()
    arguments = parser.parse_args(argv)

    if arguments.command != 'check':
        parser.error('unsupported command')

    if arguments.timeout <= 0:
        output = {
            'tool': 'buildguard',
            'version': __version__,
            'command': 'check',
            'requirements_path': arguments.requirements_path,
            'python_executable': arguments.python_executable,
            'status': 'fail',
            'exit_code': 2,
            'elapsed_seconds': 0.0,
            'venv_path': '',
            'pip_exit_code': None,
            'failing_package_hint': None,
            'failing_package_hint_is_best_effort': False,
            'failure_category': None,
            'failure_detail': None,
            'suggested_fixes': [],
            'available_versions': [],
            'available_versions_more_count': 0,
            'available_versions_query_error': None,
            'error_tail': [],
            'stdout_tail': [],
            'stderr_tail': [],
            'tool_error': True,
            'tool_error_message': '--timeout must be greater than 0',
        }
        if arguments.emit_json:
            sys.stdout.write(json.dumps(output, indent=2) + '\n')
        else:
            sys.stdout.write('buildguard check: invalid arguments\n\nFAIL\n\n--timeout must be greater than 0\n\n')
            _maybe_write_license_notice()
        return 2
    if arguments.no_upgrade_tools and (
        arguments.pip_version
        or arguments.setuptools_version
        or arguments.wheel_version
    ):
        output = {
            'tool': 'buildguard',
            'version': __version__,
            'command': 'check',
            'requirements_path': arguments.requirements_path,
            'python_executable': arguments.python_executable,
            'status': 'fail',
            'exit_code': 2,
            'elapsed_seconds': 0.0,
            'venv_path': '',
            'pip_exit_code': None,
            'failing_package_hint': None,
            'failing_package_hint_is_best_effort': False,
            'failure_category': None,
            'failure_detail': None,
            'suggested_fixes': [],
            'available_versions': [],
            'available_versions_more_count': 0,
            'available_versions_query_error': None,
            'error_tail': [],
            'stdout_tail': [],
            'stderr_tail': [],
            'tool_error': True,
            'tool_error_message': '--pip-version/--setuptools-version/--wheel-version require tool upgrades; remove --no-upgrade-tools',
        }
        if arguments.emit_json:
            sys.stdout.write(json.dumps(output, indent=2) + '\n')
        else:
            sys.stdout.write('buildguard check: invalid arguments\n\nFAIL\n\n')
            sys.stdout.write('--pip-version/--setuptools-version/--wheel-version require tool upgrades; remove --no-upgrade-tools\n')
            sys.stdout.write('\n')
            _maybe_write_license_notice()
        return 2

    show_progress = (
        not arguments.emit_json
        and sys.stderr.isatty()
        and os.environ.get('CI', '').strip().lower() not in ('1', 'true', 'yes')
    )
    progress_spinner = _ProgressSpinner(
        message=f'buildguard: checking {arguments.requirements_path}',
        enabled=show_progress,
    )

    try:
        progress_spinner.start()
        result = run_check(
            requirements_path=arguments.requirements_path,
            python_executable=arguments.python_executable,
            timeout_seconds=arguments.timeout,
            keep_venv=arguments.keep_venv,
            upgrade_tools=not arguments.no_upgrade_tools,
            pip_version=arguments.pip_version,
            setuptools_version=arguments.setuptools_version,
            wheel_version=arguments.wheel_version,
            use_python_module_pip=arguments.python_m_pip,
            show_available_versions=arguments.show_available_versions,
        )
    except Exception as unexpected_error:
        logger.info('unexpected top-level error: %s', unexpected_error)
        if arguments.emit_json:
            sys.stdout.write(
                json.dumps(
                    {
                        'tool': 'buildguard',
                        'version': __version__,
                        'command': 'check',
                        'requirements_path': arguments.requirements_path,
                        'python_executable': arguments.python_executable,
                        'status': 'fail',
                        'exit_code': 2,
                        'elapsed_seconds': 0.0,
                        'venv_path': '',
                        'pip_exit_code': None,
                        'failing_package_hint': None,
                        'failing_package_hint_is_best_effort': False,
                        'failure_category': None,
                        'failure_detail': None,
                        'suggested_fixes': [],
                        'available_versions': [],
                        'available_versions_more_count': 0,
                        'available_versions_query_error': None,
                        'error_tail': [],
                        'stdout_tail': [],
                        'stderr_tail': [],
                        'tool_error': True,
                        'tool_error_message': str(unexpected_error),
                    },
                    indent=2,
                )
                + '\n'
            )
        else:
            sys.stdout.write('buildguard check: fatal error\n\nFAIL\n\n')
            sys.stdout.write(f'{unexpected_error}\n\n')
            _maybe_write_license_notice()
        return 2
    finally:
        progress_spinner.stop()

    if arguments.emit_json:
        sys.stdout.write(format_json_report(result) + '\n')
    else:
        sys.stdout.write(
            format_text_report(
                result,
                include_verbose_errors=arguments.verbose_errors,
            )
            + '\n'
        )
    return result.exit_code


if __name__ == '__main__':
    raise SystemExit(main())



from __future__ import annotations

import json
from typing import Any, Dict

from buildguard import __version__
from buildguard.licensing import LICENSE_NOTICE, should_show_license_notice
from buildguard.models import CheckResult


def _format_elapsed_seconds(elapsed_seconds: float) -> str:
    return f'{elapsed_seconds:.1f}'


def _append_license_notice(lines: list[str]) -> None:
    if not should_show_license_notice():
        return
    lines.extend([
        '',
        LICENSE_NOTICE,
    ])


def format_text_report(
    result: CheckResult,
    include_verbose_errors: bool = False,
) -> str:
    lines = [
        f'buildguard check: {result.requirements_path}',
        f'python: {result.python_executable}',
        f'venv: {result.venv_path}',
        '',
    ]

    if result.status == 'pass':
        lines.extend([
            'PASS',
            '',
            'requirements installed successfully in a clean environment',
            '',
            'summary:',
            f'elapsed_seconds={_format_elapsed_seconds(result.elapsed_seconds)}',
            f'pip_exit_code={result.pip_exit_code}',
        ])
        _append_license_notice(lines)
        return '\n'.join(lines)

    lines.extend(['FAIL', ''])

    if result.tool_error:
        lines.append('buildguard encountered a fatal tool/runtime error')
        lines.append('')
        if result.tool_error_message:
            lines.append(result.tool_error_message)
            lines.append('')
    else:
        pip_exit_code = result.pip_exit_code if result.pip_exit_code is not None else -1
        lines.append(f'pip install -r {result.requirements_path} exited with code {pip_exit_code}')
        lines.append('')
        if result.failing_package_hint:
            if result.failing_package_hint_is_best_effort:
                lines.append('likely failing dependency (best effort):')
            else:
                lines.append('likely failing dependency:')
            lines.append(result.failing_package_hint)
            lines.append('')
        if result.error_tail:
            lines.append('pip error summary:')
            for line in result.error_tail:
                lines.append(line)
            lines.append('')
        if result.failure_detail:
            lines.append('diagnosis:')
            lines.append(result.failure_detail)
            lines.append('')
        if result.suggested_fixes:
            lines.append('what to try:')
            for fix in result.suggested_fixes:
                lines.append(f'- {fix}')
            lines.append('')
        if result.available_versions:
            versions_line = ', '.join(result.available_versions)
            if result.available_versions_more_count > 0:
                versions_line = f'{versions_line}, ... (+{result.available_versions_more_count} more)'
            lines.append('available versions (latest first):')
            lines.append(versions_line)
            lines.append('')
        if result.available_versions_query_error:
            lines.append('available versions:')
            lines.append(f'unavailable ({result.available_versions_query_error})')
            lines.append('')
        if include_verbose_errors:
            if result.stderr_tail:
                lines.append('verbose pip stderr tail:')
                for line in result.stderr_tail:
                    lines.append(f'  {line}')
                lines.append('')
            if result.stdout_tail:
                lines.append('verbose pip stdout tail:')
                for line in result.stdout_tail:
                    lines.append(f'  {line}')
                lines.append('')

    lines.extend([
        'summary:',
        f'elapsed_seconds={_format_elapsed_seconds(result.elapsed_seconds)}',
        f'pip_exit_code={result.pip_exit_code}',
    ])
    _append_license_notice(lines)
    return '\n'.join(lines)


def format_json_report(result: CheckResult) -> str:
    payload: Dict[str, Any] = {
        'tool': 'buildguard',
        'version': __version__,
        'command': 'check',
        'requirements_path': result.requirements_path,
        'python_executable': result.python_executable,
        'status': result.status,
        'exit_code': result.exit_code,
        'elapsed_seconds': round(result.elapsed_seconds, 3),
        'venv_path': result.venv_path,
        'pip_exit_code': result.pip_exit_code,
        'failing_package_hint': result.failing_package_hint,
        'failing_package_hint_is_best_effort': result.failing_package_hint_is_best_effort,
        'failure_category': result.failure_category,
        'failure_detail': result.failure_detail,
        'suggested_fixes': list(result.suggested_fixes),
        'available_versions': list(result.available_versions),
        'available_versions_more_count': result.available_versions_more_count,
        'available_versions_query_error': result.available_versions_query_error,
        'error_tail': list(result.error_tail),
        'stdout_tail': list(result.stdout_tail),
        'stderr_tail': list(result.stderr_tail),
        'tool_error': result.tool_error,
    }
    if result.tool_error and result.tool_error_message:
        payload['tool_error_message'] = result.tool_error_message
    return json.dumps(payload, indent=2)



from __future__ import annotations

from dataclasses import dataclass
from typing import Optional, Tuple


@dataclass(frozen=True)
class CommandResult:
    args: Tuple[str, ...]
    return_code: int
    stdout: str
    stderr: str
    timed_out: bool


@dataclass(frozen=True)
class CheckResult:
    requirements_path: str
    python_executable: str
    venv_path: str
    status: str
    exit_code: int
    tool_error: bool
    tool_error_message: Optional[str]
    pip_exit_code: Optional[int]
    elapsed_seconds: float
    failing_package_hint: Optional[str]
    failing_package_hint_is_best_effort: bool
    failure_category: Optional[str]
    failure_detail: Optional[str]
    suggested_fixes: Tuple[str, ...]
    available_versions: Tuple[str, ...]
    available_versions_more_count: int
    available_versions_query_error: Optional[str]
    error_tail: Tuple[str, ...]
    stdout_tail: Tuple[str, ...]
    stderr_tail: Tuple[str, ...]



from __future__ import annotations

import logging
import signal
import shutil
import tempfile
import time
from pathlib import Path
from typing import Optional

from buildguard.models import CheckResult
from buildguard.parsing import (
    extract_package_name_from_requirement_spec,
    extract_error_tail,
    extract_failing_package_hint,
    extract_stream_tail,
    infer_failure_diagnostics,
    parse_available_versions_from_pip_index_output,
)
from buildguard.process import run_command
from buildguard.venvtools import create_virtual_environment, get_venv_executables

logger = logging.getLogger(__name__)
_MAX_DISPLAY_AVAILABLE_VERSIONS = 8


def _describe_command_failure(command_result) -> str:
    message = command_result.stderr.strip() or command_result.stdout.strip()
    if message:
        return message

    command_text = ' '.join(command_result.args)
    return_code = command_result.return_code
    if return_code < 0:
        signal_number = -return_code
        try:
            signal_name = signal.Signals(signal_number).name
            return f"command '{command_text}' terminated by signal {signal_name} ({signal_number})"
        except ValueError:
            return f"command '{command_text}' terminated by signal {signal_number}"
    return f"command '{command_text}' exited with code {return_code}"


def _find_pip_colorama_win32_module(venv_path: Path) -> Optional[Path]:
    if not (venv_path / 'lib').exists():
        return None
    for candidate in sorted((venv_path / 'lib').glob('python*/site-packages/pip/_vendor/colorama/win32.py')):
        if candidate.is_file():
            return candidate
    return None


def _apply_non_windows_pip36_ctypes_workaround(venv_path: Path) -> None:
    win32_module_path = _find_pip_colorama_win32_module(venv_path)
    if win32_module_path is None:
        return

    original_text = win32_module_path.read_text(encoding='utf-8')
    guard_line = "if os.name != 'nt':"
    if guard_line in original_text:
        return

    marker = 'STDERR = -12\n\n'
    if marker not in original_text:
        return

    guard_text = (
        "import os\n"
        "if os.name != 'nt':\n"
        "    raise ImportError('win32 APIs are unavailable on non-Windows hosts')\n\n"
    )
    patched_text = original_text.replace(marker, marker + guard_text, 1)
    win32_module_path.write_text(patched_text, encoding='utf-8')


def run_check(
    requirements_path: str,
    python_executable: str,
    timeout_seconds: int,
    keep_venv: bool,
    upgrade_tools: bool,
    pip_version: Optional[str] = None,
    setuptools_version: Optional[str] = None,
    wheel_version: Optional[str] = None,
    use_python_module_pip: bool = False,
    show_available_versions: bool = False,
) -> CheckResult:
    start_time = time.monotonic()
    requirements_file_path = Path(requirements_path)

    if not requirements_file_path.is_file():
        elapsed_seconds = round(time.monotonic() - start_time, 3)
        return CheckResult(
            requirements_path=str(requirements_file_path),
            python_executable=python_executable,
            venv_path='',
            status='fail',
            exit_code=2,
            tool_error=True,
            tool_error_message=f'ERROR: requirements file not found: {requirements_file_path}',
            pip_exit_code=None,
            elapsed_seconds=elapsed_seconds,
            failing_package_hint=None,
            failing_package_hint_is_best_effort=False,
            failure_category=None,
            failure_detail=None,
            suggested_fixes=tuple(),
            available_versions=tuple(),
            available_versions_more_count=0,
            available_versions_query_error=None,
            error_tail=tuple(),
            stdout_tail=tuple(),
            stderr_tail=tuple(),
        )

    temp_directory = tempfile.mkdtemp(prefix='buildguard-')
    venv_path = Path(temp_directory)

    try:
        logger.info('creating virtual environment at %s', venv_path)
        create_virtual_environment(venv_path=venv_path, python_executable=python_executable)
        venv_python_executable, pip_executable = get_venv_executables(venv_path)
        _apply_non_windows_pip36_ctypes_workaround(venv_path)
        pip_command_prefix = (
            [str(venv_python_executable), '-m', 'pip']
            if use_python_module_pip
            else [str(pip_executable)]
        )

        if upgrade_tools:
            logger.info('upgrading pip, setuptools, and wheel')
            pip_upgrade_targets = [
                f'pip=={pip_version}' if pip_version else 'pip',
                f'setuptools=={setuptools_version}' if setuptools_version else 'setuptools',
                f'wheel=={wheel_version}' if wheel_version else 'wheel',
            ]
            upgrade_result = run_command([
                *pip_command_prefix,
                'install',
                '--upgrade',
                *pip_upgrade_targets,
            ])
            if upgrade_result.return_code != 0:
                elapsed_seconds = round(time.monotonic() - start_time, 3)
                tool_error_message = _describe_command_failure(upgrade_result)
                return CheckResult(
                    requirements_path=str(requirements_file_path),
                    python_executable=python_executable,
                    venv_path=str(venv_path),
                    status='fail',
                    exit_code=2,
                    tool_error=True,
                    tool_error_message=f'failed to upgrade tools: {tool_error_message}',
                    pip_exit_code=None,
                    elapsed_seconds=elapsed_seconds,
                    failing_package_hint=None,
                    failing_package_hint_is_best_effort=False,
                    failure_category=None,
                    failure_detail=None,
                    suggested_fixes=tuple(),
                    available_versions=tuple(),
                    available_versions_more_count=0,
                    available_versions_query_error=None,
                    error_tail=tuple(),
                    stdout_tail=extract_stream_tail(upgrade_result.stdout),
                    stderr_tail=extract_stream_tail(upgrade_result.stderr),
                )

        logger.info('running pip install -r %s', requirements_file_path)
        pip_install_result = run_command(
            [*pip_command_prefix, 'install', '-r', str(requirements_file_path)],
            timeout_seconds=timeout_seconds,
        )

        elapsed_seconds = round(time.monotonic() - start_time, 3)
        if pip_install_result.timed_out:
            return CheckResult(
                requirements_path=str(requirements_file_path),
                python_executable=python_executable,
                venv_path=str(venv_path),
                status='fail',
                exit_code=2,
                tool_error=True,
                tool_error_message='pip install timed out',
                pip_exit_code=None,
                elapsed_seconds=elapsed_seconds,
                failing_package_hint=None,
                failing_package_hint_is_best_effort=False,
                failure_category=None,
                failure_detail=None,
                suggested_fixes=tuple(),
                available_versions=tuple(),
                available_versions_more_count=0,
                available_versions_query_error=None,
                error_tail=extract_error_tail(pip_install_result.stdout, pip_install_result.stderr),
                stdout_tail=extract_stream_tail(pip_install_result.stdout),
                stderr_tail=extract_stream_tail(pip_install_result.stderr),
            )

        status = 'pass' if pip_install_result.return_code == 0 else 'fail'
        exit_code = 0 if status == 'pass' else 1

        combined_output_text = pip_install_result.stdout + '\n' + pip_install_result.stderr
        failing_package_hint, failing_package_hint_is_best_effort = extract_failing_package_hint(combined_output_text)
        failure_category, failure_detail, suggested_fixes = infer_failure_diagnostics(
            output_text=combined_output_text,
            failing_package_hint=failing_package_hint,
        )
        error_tail = extract_error_tail(pip_install_result.stdout, pip_install_result.stderr)
        available_versions = tuple()
        available_versions_more_count = 0
        available_versions_query_error = None

        if (
            show_available_versions
            and status == 'fail'
            and failure_category == 'missing_distribution'
            and failing_package_hint
        ):
            package_name = extract_package_name_from_requirement_spec(failing_package_hint)
            versions_result = run_command(
                [*pip_command_prefix, 'index', 'versions', package_name],
                timeout_seconds=min(timeout_seconds, 30),
            )
            if versions_result.return_code == 0:
                parsed_versions = parse_available_versions_from_pip_index_output(
                    versions_result.stdout + '\n' + versions_result.stderr
                )
                available_versions = parsed_versions[:_MAX_DISPLAY_AVAILABLE_VERSIONS]
                available_versions_more_count = max(0, len(parsed_versions) - len(available_versions))
            else:
                available_versions_query_error = (
                    versions_result.stderr.strip()
                    or versions_result.stdout.strip()
                    or 'unable to fetch available versions'
                )

        return CheckResult(
            requirements_path=str(requirements_file_path),
            python_executable=python_executable,
            venv_path=str(venv_path),
            status=status,
            exit_code=exit_code,
            tool_error=False,
            tool_error_message=None,
            pip_exit_code=0 if status == 'pass' else pip_install_result.return_code,
            elapsed_seconds=elapsed_seconds,
            failing_package_hint=failing_package_hint,
            failing_package_hint_is_best_effort=failing_package_hint_is_best_effort,
            failure_category=failure_category,
            failure_detail=failure_detail,
            suggested_fixes=suggested_fixes,
            available_versions=available_versions,
            available_versions_more_count=available_versions_more_count,
            available_versions_query_error=available_versions_query_error,
            error_tail=error_tail,
            stdout_tail=extract_stream_tail(pip_install_result.stdout),
            stderr_tail=extract_stream_tail(pip_install_result.stderr),
        )

    except Exception as unexpected_error:
        elapsed_seconds = round(time.monotonic() - start_time, 3)
        return CheckResult(
            requirements_path=str(requirements_file_path),
            python_executable=python_executable,
            venv_path=str(venv_path),
            status='fail',
            exit_code=2,
            tool_error=True,
            tool_error_message=str(unexpected_error),
            pip_exit_code=None,
            elapsed_seconds=elapsed_seconds,
            failing_package_hint=None,
            failing_package_hint_is_best_effort=False,
            failure_category=None,
            failure_detail=None,
            suggested_fixes=tuple(),
            available_versions=tuple(),
            available_versions_more_count=0,
            available_versions_query_error=None,
            error_tail=tuple(),
            stdout_tail=tuple(),
            stderr_tail=tuple(),
        )
    finally:
        if not keep_venv and venv_path.exists():
            shutil.rmtree(venv_path, ignore_errors=True)



from __future__ import annotations

import os

LICENSE_NOTICE = (
    'Note: buildguard is free for evaluation/personal use. '
    'Commercial teams should purchase a license. '
    'Info: https://hefftools.dev/buildguard'
)

DEFAULT_SHOW_LICENSE_NOTICE = True
HIDE_LICENSE_NOTICE_ENV_VAR = 'BUILDGUARD_HIDE_LICENSE_NOTICE'


def should_show_license_notice() -> bool:
    override = os.environ.get(HIDE_LICENSE_NOTICE_ENV_VAR)
    if override is not None:
        return override.strip().lower() not in {'1', 'true', 'yes', 'on'}
    return DEFAULT_SHOW_LICENSE_NOTICE



from __future__ import annotations

import subprocess
from pathlib import Path
from typing import Optional, Sequence

from buildguard.models import CommandResult


def _normalize_output(maybe_bytes: object) -> str:
    if maybe_bytes is None:
        return ''
    if isinstance(maybe_bytes, bytes):
        return maybe_bytes.decode(errors='replace')
    if isinstance(maybe_bytes, str):
        return maybe_bytes
    return str(maybe_bytes)


def run_command(
    args: Sequence[str],
    timeout_seconds: Optional[int] = None,
    cwd: Optional[Path] = None,
) -> CommandResult:
    try:
        completed_process = subprocess.run(
            list(args),
            capture_output=True,
            text=True,
            errors='replace',
            timeout=timeout_seconds,
            cwd=str(cwd) if cwd is not None else None,
            check=False,
        )
        return CommandResult(
            args=tuple(str(argument) for argument in args),
            return_code=completed_process.returncode,
            stdout=completed_process.stdout,
            stderr=completed_process.stderr,
            timed_out=False,
        )
    except subprocess.TimeoutExpired as timeout_error:
        return CommandResult(
            args=tuple(str(argument) for argument in args),
            return_code=-1,
            stdout=_normalize_output(timeout_error.stdout),
            stderr=_normalize_output(timeout_error.stderr),
            timed_out=True,
        )
